@@ -1,5 +1,4 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function
 import itertools
 import pickle
 from weakref import ref
@@ -10,45 +9,15 @@ import six
 from datetime import datetime
 
 import numpy as np
-from numpy.testing.utils import (assert_array_equal, assert_approx_equal,
-                                 assert_array_almost_equal)
-from nose.tools import (assert_equal, assert_not_equal, raises, assert_true,
-                        assert_raises)
+from numpy.testing import (assert_array_equal, assert_approx_equal,
+                           assert_array_almost_equal)
+import pytest
 
 import matplotlib.cbook as cbook
 import matplotlib.colors as mcolors
 from matplotlib.cbook import delete_masked_points as dmp
 
 
-def test_is_string_like():
-    y = np.arange(10)
-    assert_equal(cbook.is_string_like(y), False)
-    y.shape = 10, 1
-    assert_equal(cbook.is_string_like(y), False)
-    y.shape = 1, 10
-    assert_equal(cbook.is_string_like(y), False)
-
-    assert cbook.is_string_like("hello world")
-    assert_equal(cbook.is_string_like(10), False)
-
-    y = ['a', 'b', 'c']
-    assert_equal(cbook.is_string_like(y), False)
-
-    y = np.array(y)
-    assert_equal(cbook.is_string_like(y), False)
-
-    y = np.array(y, dtype=object)
-    assert cbook.is_string_like(y)
-
-
-def test_is_sequence_of_strings():
-    y = ['a', 'b', 'c']
-    assert cbook.is_sequence_of_strings(y)
-
-    y = np.array(y, dtype=object)
-    assert cbook.is_sequence_of_strings(y)
-
-
 def test_is_hashable():
     s = 'string'
     assert cbook.is_hashable(s)
@@ -59,22 +28,24 @@ def test_is_hashable():
 
 def test_restrict_dict():
     d = {'foo': 'bar', 1: 2}
-    d1 = cbook.restrict_dict(d, ['foo', 1])
-    assert_equal(d1, d)
-    d2 = cbook.restrict_dict(d, ['bar', 2])
-    assert_equal(d2, {})
-    d3 = cbook.restrict_dict(d, {'foo': 1})
-    assert_equal(d3, {'foo': 'bar'})
-    d4 = cbook.restrict_dict(d, {})
-    assert_equal(d4, {})
-    d5 = cbook.restrict_dict(d, set(['foo', 2]))
-    assert_equal(d5, {'foo': 'bar'})
+    with pytest.warns(cbook.deprecation.MatplotlibDeprecationWarning) as rec:
+        d1 = cbook.restrict_dict(d, ['foo', 1])
+        assert d1 == d
+        d2 = cbook.restrict_dict(d, ['bar', 2])
+        assert d2 == {}
+        d3 = cbook.restrict_dict(d, {'foo': 1})
+        assert d3 == {'foo': 'bar'}
+        d4 = cbook.restrict_dict(d, {})
+        assert d4 == {}
+        d5 = cbook.restrict_dict(d, {'foo', 2})
+        assert d5 == {'foo': 'bar'}
+    assert len(rec) == 5
     # check that d was not modified
-    assert_equal(d, {'foo': 'bar', 1: 2})
+    assert d == {'foo': 'bar', 1: 2}
 
 
 class Test_delete_masked_points(object):
-    def setUp(self):
+    def setup_method(self):
         self.mask1 = [False, False, True, True, False, False]
         self.arr0 = np.arange(1.0, 7.0)
         self.arr1 = [1, 2, 3, np.nan, np.nan, 6]
@@ -89,9 +60,9 @@ class Test_delete_masked_points(object):
         self.arr_colors = ['r', 'g', 'b', 'c', 'm', 'y']
         self.arr_rgba = mcolors.to_rgba_array(self.arr_colors)
 
-    @raises(ValueError)
     def test_bad_first_arg(self):
-        dmp('a string', self.arr0)
+        with pytest.raises(ValueError):
+            dmp('a string', self.arr0)
 
     def test_string_seq(self):
         actual = dmp(self.arr_s, self.arr1)
@@ -117,14 +88,6 @@ class Test_delete_masked_points(object):
         assert_array_equal(actual[1], expected[1])
 
 
-def test_allequal():
-    assert(cbook.allequal([1, 1, 1]))
-    assert(not cbook.allequal([1, 1, 0]))
-    assert(cbook.allequal([]))
-    assert(cbook.allequal(('a', 'a')))
-    assert(not cbook.allequal(('a', 'b')))
-
-
 class Test_boxplot_stats(object):
     def setup(self):
         np.random.seed(937)
@@ -177,102 +140,65 @@ class Test_boxplot_stats(object):
         }
 
     def test_form_main_list(self):
-        assert_true(isinstance(self.std_results, list))
+        assert isinstance(self.std_results, list)
 
     def test_form_each_dict(self):
         for res in self.std_results:
-            assert_true(isinstance(res, dict))
+            assert isinstance(res, dict)
 
     def test_form_dict_keys(self):
         for res in self.std_results:
-            keys = sorted(list(res.keys()))
-            for key in keys:
-                assert_true(key in self.known_keys)
+            assert set(res) <= set(self.known_keys)
 
     def test_results_baseline(self):
         res = self.std_results[0]
-        for key in list(self.known_nonbootstrapped_res.keys()):
-            if key != 'fliers':
-                assert_statement = assert_approx_equal
-            else:
-                assert_statement = assert_array_almost_equal
-
-            assert_statement(
-                res[key],
-                self.known_nonbootstrapped_res[key]
-            )
+        for key, value in self.known_nonbootstrapped_res.items():
+            assert_array_almost_equal(res[key], value)
 
     def test_results_bootstrapped(self):
         results = cbook.boxplot_stats(self.data, bootstrap=10000)
         res = results[0]
-        for key in list(self.known_bootstrapped_ci.keys()):
-            assert_approx_equal(
-                res[key],
-                self.known_bootstrapped_ci[key]
-            )
+        for key, value in self.known_bootstrapped_ci.items():
+            assert_approx_equal(res[key], value)
 
     def test_results_whiskers_float(self):
         results = cbook.boxplot_stats(self.data, whis=3)
         res = results[0]
-        for key in list(self.known_whis3_res.keys()):
-            if key != 'fliers':
-                assert_statement = assert_approx_equal
-            else:
-                assert_statement = assert_array_almost_equal
-
-            assert_statement(
-                res[key],
-                self.known_whis3_res[key]
-            )
+        for key, value in self.known_whis3_res.items():
+            assert_array_almost_equal(res[key], value)
 
     def test_results_whiskers_range(self):
         results = cbook.boxplot_stats(self.data, whis='range')
         res = results[0]
-        for key in list(self.known_res_range.keys()):
-            if key != 'fliers':
-                assert_statement = assert_approx_equal
-            else:
-                assert_statement = assert_array_almost_equal
-
-            assert_statement(
-                res[key],
-                self.known_res_range[key]
-            )
+        for key, value in self.known_res_range.items():
+            assert_array_almost_equal(res[key], value)
 
     def test_results_whiskers_percentiles(self):
         results = cbook.boxplot_stats(self.data, whis=[5, 95])
         res = results[0]
-        for key in list(self.known_res_percentiles.keys()):
-            if key != 'fliers':
-                assert_statement = assert_approx_equal
-            else:
-                assert_statement = assert_array_almost_equal
-
-            assert_statement(
-                res[key],
-                self.known_res_percentiles[key]
-            )
+        for key, value in self.known_res_percentiles.items():
+            assert_array_almost_equal(res[key], value)
 
     def test_results_withlabels(self):
         labels = ['Test1', 2, 'ardvark', 4]
         results = cbook.boxplot_stats(self.data, labels=labels)
         res = results[0]
         for lab, res in zip(labels, results):
-            assert_equal(res['label'], lab)
+            assert res['label'] == lab
 
         results = cbook.boxplot_stats(self.data)
         for res in results:
-            assert('label' not in res)
+            assert 'label' not in res
 
-    @raises(ValueError)
     def test_label_error(self):
         labels = [1, 2]
-        results = cbook.boxplot_stats(self.data, labels=labels)
+        with pytest.raises(ValueError):
+            results = cbook.boxplot_stats(self.data, labels=labels)
 
-    @raises(ValueError)
     def test_bad_dims(self):
         data = np.random.normal(size=(34, 34, 34))
-        results = cbook.boxplot_stats(data)
+        with pytest.raises(ValueError):
+            results = cbook.boxplot_stats(data)
 
     def test_boxplot_stats_autorange_false(self):
         x = np.zeros(shape=140)
@@ -280,12 +206,12 @@ class Test_boxplot_stats(object):
         bstats_false = cbook.boxplot_stats(x, autorange=False)
         bstats_true = cbook.boxplot_stats(x, autorange=True)
 
-        assert_equal(bstats_false[0]['whislo'], 0)
-        assert_equal(bstats_false[0]['whishi'], 0)
+        assert bstats_false[0]['whislo'] == 0
+        assert bstats_false[0]['whishi'] == 0
         assert_array_almost_equal(bstats_false[0]['fliers'], [-25, 25])
 
-        assert_equal(bstats_true[0]['whislo'], -25)
-        assert_equal(bstats_true[0]['whishi'], 25)
+        assert bstats_true[0]['whislo'] == -25
+        assert bstats_true[0]['whishi'] == 25
         assert_array_almost_equal(bstats_true[0]['fliers'], [])
 
 
@@ -298,12 +224,12 @@ class Test_callback_registry(object):
         return self.callbacks.connect(s, func)
 
     def is_empty(self):
-        assert_equal(self.callbacks._func_cid_map, {})
-        assert_equal(self.callbacks.callbacks, {})
+        assert self.callbacks._func_cid_map == {}
+        assert self.callbacks.callbacks == {}
 
     def is_not_empty(self):
-        assert_not_equal(self.callbacks._func_cid_map, {})
-        assert_not_equal(self.callbacks.callbacks, {})
+        assert self.callbacks._func_cid_map != {}
+        assert self.callbacks.callbacks != {}
 
     def test_callback_complete(self):
         # ensure we start with an empty registry
@@ -314,15 +240,15 @@ class Test_callback_registry(object):
 
         # test that we can add a callback
         cid1 = self.connect(self.signal, mini_me.dummy)
-        assert_equal(type(cid1), int)
+        assert type(cid1) == int
         self.is_not_empty()
 
         # test that we don't add a second callback
         cid2 = self.connect(self.signal, mini_me.dummy)
-        assert_equal(cid1, cid2)
+        assert cid1 == cid2
         self.is_not_empty()
-        assert_equal(len(self.callbacks._func_cid_map), 1)
-        assert_equal(len(self.callbacks.callbacks), 1)
+        assert len(self.callbacks._func_cid_map) == 1
+        assert len(self.callbacks.callbacks) == 1
 
         del mini_me
 
@@ -337,65 +263,112 @@ class Test_callback_registry(object):
                        "callbacks")
 
 
-def _kwarg_norm_helper(inp, expected, kwargs_to_norm, warn_count=0):
+def raising_cb_reg(func):
+    class TestException(Exception):
+        pass
 
+    def raising_function():
+        raise RuntimeError
+
+    def transformer(excp):
+        if isinstance(excp, RuntimeError):
+            raise TestException
+        raise excp
+
+    # default behavior
+    cb = cbook.CallbackRegistry()
+    cb.connect('foo', raising_function)
+
+    # old default
+    cb_old = cbook.CallbackRegistry(exception_handler=None)
+    cb_old.connect('foo', raising_function)
+
+    # filter
+    cb_filt = cbook.CallbackRegistry(exception_handler=transformer)
+    cb_filt.connect('foo', raising_function)
+
+    return pytest.mark.parametrize('cb, excp',
+                                   [[cb, None],
+                                    [cb_old, RuntimeError],
+                                    [cb_filt, TestException]])(func)
+
+
+@raising_cb_reg
+def test_callbackregistry_process_exception(cb, excp):
+    if excp is not None:
+        with pytest.raises(excp):
+            cb.process('foo')
+    else:
+        cb.process('foo')
+
+
+def test_sanitize_sequence():
+    d = {'a': 1, 'b': 2, 'c': 3}
+    k = ['a', 'b', 'c']
+    v = [1, 2, 3]
+    i = [('a', 1), ('b', 2), ('c', 3)]
+    assert k == sorted(cbook.sanitize_sequence(d.keys()))
+    assert v == sorted(cbook.sanitize_sequence(d.values()))
+    assert i == sorted(cbook.sanitize_sequence(d.items()))
+    assert i == cbook.sanitize_sequence(i)
+    assert k == cbook.sanitize_sequence(k)
+
+
+fail_mapping = (
+    ({'a': 1}, {'forbidden': ('a')}),
+    ({'a': 1}, {'required': ('b')}),
+    ({'a': 1, 'b': 2}, {'required': ('a'), 'allowed': ()})
+)
+
+warn_passing_mapping = (
+    ({'a': 1, 'b': 2}, {'a': 1}, {'alias_mapping': {'a': ['b']}}, 1),
+    ({'a': 1, 'b': 2}, {'a': 1},
+     {'alias_mapping': {'a': ['b']}, 'allowed': ('a',)}, 1),
+    ({'a': 1, 'b': 2}, {'a': 2}, {'alias_mapping': {'a': ['a', 'b']}}, 1),
+    ({'a': 1, 'b': 2, 'c': 3}, {'a': 1, 'c': 3},
+     {'alias_mapping': {'a': ['b']}, 'required': ('a', )}, 1),
+)
+
+pass_mapping = (
+    ({'a': 1, 'b': 2}, {'a': 1, 'b': 2}, {}),
+    ({'b': 2}, {'a': 2}, {'alias_mapping': {'a': ['a', 'b']}}),
+    ({'b': 2}, {'a': 2},
+     {'alias_mapping': {'a': ['b']}, 'forbidden': ('b', )}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3},
+     {'required': ('a', ), 'allowed': ('c', )}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3},
+     {'required': ('a', 'c'), 'allowed': ('c', )}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3},
+     {'required': ('a', 'c'), 'allowed': ('a', 'c')}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3},
+     {'required': ('a', 'c'), 'allowed': ()}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', 'c')}),
+    ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'allowed': ('a', 'c')}),
+)
+
+
+@pytest.mark.parametrize('inp, kwargs_to_norm', fail_mapping)
+def test_normalize_kwargs_fail(inp, kwargs_to_norm):
+    with pytest.raises(TypeError):
+        cbook.normalize_kwargs(inp, **kwargs_to_norm)
+
+
+@pytest.mark.parametrize('inp, expected, kwargs_to_norm, warn_count',
+                         warn_passing_mapping)
+def test_normalize_kwargs_warn(inp, expected, kwargs_to_norm, warn_count):
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter("always")
         assert expected == cbook.normalize_kwargs(inp, **kwargs_to_norm)
         assert len(w) == warn_count
 
 
-def _kwarg_norm_fail_helper(inp, kwargs_to_norm):
-    assert_raises(TypeError, cbook.normalize_kwargs, inp, **kwargs_to_norm)
-
-
-def test_normalize_kwargs():
-    fail_mapping = (
-        ({'a': 1}, {'forbidden': ('a')}),
-        ({'a': 1}, {'required': ('b')}),
-        ({'a': 1, 'b': 2}, {'required': ('a'), 'allowed': ()})
-    )
-
-    for inp, kwargs in fail_mapping:
-        yield _kwarg_norm_fail_helper, inp, kwargs
-
-    warn_passing_mapping = (
-        ({'a': 1, 'b': 2}, {'a': 1}, {'alias_mapping': {'a': ['b']}}, 1),
-        ({'a': 1, 'b': 2}, {'a': 1}, {'alias_mapping': {'a': ['b']},
-                                      'allowed': ('a',)}, 1),
-        ({'a': 1, 'b': 2}, {'a': 2}, {'alias_mapping': {'a': ['a', 'b']}}, 1),
-
-        ({'a': 1, 'b': 2, 'c': 3}, {'a': 1, 'c': 3},
-         {'alias_mapping': {'a': ['b']}, 'required': ('a', )}, 1),
-
-    )
-
-    for inp, exp, kwargs, wc in warn_passing_mapping:
-        yield _kwarg_norm_helper, inp, exp, kwargs, wc
-
-    pass_mapping = (
-        ({'a': 1, 'b': 2}, {'a': 1, 'b': 2}, {}),
-        ({'b': 2}, {'a': 2}, {'alias_mapping': {'a': ['a', 'b']}}),
-        ({'b': 2}, {'a': 2}, {'alias_mapping': {'a': ['b']},
-                              'forbidden': ('b', )}),
-
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', ),
-                                              'allowed': ('c', )}),
-
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', 'c'),
-                                              'allowed': ('c', )}),
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', 'c'),
-                                              'allowed': ('a', 'c')}),
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', 'c'),
-                                              'allowed': ()}),
-
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'required': ('a', 'c')}),
-        ({'a': 1, 'c': 3}, {'a': 1, 'c': 3}, {'allowed': ('a', 'c')}),
-
-    )
-
-    for inp, exp, kwargs in pass_mapping:
-        yield _kwarg_norm_helper, inp, exp, kwargs
+@pytest.mark.parametrize('inp, expected, kwargs_to_norm',
+                         pass_mapping)
+def test_normalize_kwargs_pass(inp, expected, kwargs_to_norm):
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        assert expected == cbook.normalize_kwargs(inp, **kwargs_to_norm)
+        assert len(w) == 0
 
 
 def test_to_prestep():
@@ -418,6 +391,11 @@ def test_to_prestep():
     assert_array_equal(y1_target, y1s)
 
 
+def test_to_prestep_empty():
+    steps = cbook.pts_to_prestep([], [])
+    assert steps.shape == (2, 0)
+
+
 def test_to_poststep():
     x = np.arange(4)
     y1 = np.arange(4)
@@ -438,6 +416,11 @@ def test_to_poststep():
     assert_array_equal(y1_target, y1s)
 
 
+def test_to_poststep_empty():
+    steps = cbook.pts_to_poststep([], [])
+    assert steps.shape == (2, 0)
+
+
 def test_to_midstep():
     x = np.arange(4)
     y1 = np.arange(4)
@@ -458,15 +441,19 @@ def test_to_midstep():
     assert_array_equal(y1_target, y1s)
 
 
-def test_step_fails():
-    assert_raises(ValueError, cbook._step_validation,
-                  np.arange(12).reshape(3, 4), 'a')
-    assert_raises(ValueError, cbook._step_validation,
-                  np.arange(12), 'a')
-    assert_raises(ValueError, cbook._step_validation,
-                  np.arange(12))
-    assert_raises(ValueError, cbook._step_validation,
-                  np.arange(12), np.arange(3))
+def test_to_midstep_empty():
+    steps = cbook.pts_to_midstep([], [])
+    assert steps.shape == (2, 0)
+
+
+@pytest.mark.parametrize(
+    "args",
+    [(np.arange(12).reshape(3, 4), 'a'),
+     (np.arange(12), 'a'),
+     (np.arange(12), np.arange(3))])
+def test_step_fails(args):
+    with pytest.raises(ValueError):
+        cbook.pts_to_prestep(*args)
 
 
 def test_grouper():
@@ -516,3 +503,64 @@ def test_flatiter():
 
     assert 0 == next(it)
     assert 1 == next(it)
+
+
+class TestFuncParser(object):
+    x_test = np.linspace(0.01, 0.5, 3)
+    validstrings = ['linear', 'quadratic', 'cubic', 'sqrt', 'cbrt',
+                    'log', 'log10', 'log2', 'x**{1.5}', 'root{2.5}(x)',
+                    'log{2}(x)',
+                    'log(x+{0.5})', 'log10(x+{0.1})', 'log{2}(x+{0.1})',
+                    'log{2}(x+{0})']
+    results = [(lambda x: x),
+               np.square,
+               (lambda x: x**3),
+               np.sqrt,
+               (lambda x: x**(1. / 3)),
+               np.log,
+               np.log10,
+               np.log2,
+               (lambda x: x**1.5),
+               (lambda x: x**(1 / 2.5)),
+               (lambda x: np.log2(x)),
+               (lambda x: np.log(x + 0.5)),
+               (lambda x: np.log10(x + 0.1)),
+               (lambda x: np.log2(x + 0.1)),
+               (lambda x: np.log2(x))]
+
+    bounded_list = [True, True, True, True, True,
+                    False, False, False, True, True,
+                    False,
+                    True, True, True,
+                    False]
+
+    @pytest.mark.parametrize("string, func",
+                             zip(validstrings, results),
+                             ids=validstrings)
+    def test_values(self, string, func):
+        func_parser = cbook._StringFuncParser(string)
+        f = func_parser.function
+        assert_array_almost_equal(f(self.x_test), func(self.x_test))
+
+    @pytest.mark.parametrize("string", validstrings, ids=validstrings)
+    def test_inverse(self, string):
+        func_parser = cbook._StringFuncParser(string)
+        f = func_parser.func_info
+        fdir = f.function
+        finv = f.inverse
+        assert_array_almost_equal(finv(fdir(self.x_test)), self.x_test)
+
+    @pytest.mark.parametrize("string", validstrings, ids=validstrings)
+    def test_get_inverse(self, string):
+        func_parser = cbook._StringFuncParser(string)
+        finv1 = func_parser.inverse
+        finv2 = func_parser.func_info.inverse
+        assert_array_almost_equal(finv1(self.x_test), finv2(self.x_test))
+
+    @pytest.mark.parametrize("string, bounded",
+                             zip(validstrings, bounded_list),
+                             ids=validstrings)
+    def test_bounded(self, string, bounded):
+        func_parser = cbook._StringFuncParser(string)
+        b = func_parser.is_bounded_0_1
+        assert_array_equal(b, bounded)
