@@ -1,6 +1,7 @@
 import numpy as np
 from matplotlib import markers
-from nose.tools import assert_raises
+
+import pytest
 
 
 def test_markers_valid():
@@ -15,5 +16,5 @@ def test_markers_invalid():
     marker_style = markers.MarkerStyle()
     mrk_array = np.array([[-0.5, 0, 1, 2, 3]])
     # Checking this does fail.
-    with assert_raises(ValueError):
+    with pytest.raises(ValueError):
         marker_style.set_marker(mrk_array)
