@@ -1,21 +1,24 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function
 
-from matplotlib.externals import six
+import six
+
+import io
 import warnings
 
 import numpy as np
 from numpy.testing import assert_almost_equal
-from nose.tools import eq_, assert_raises
-from nose.plugins.skip import SkipTest
+import pytest
 
-from matplotlib.transforms import Bbox
 import matplotlib
 import matplotlib.pyplot as plt
-from matplotlib.testing.decorators import image_comparison, cleanup
-from matplotlib.figure import Figure
-from matplotlib.text import Annotation, Text
-from matplotlib.backends.backend_agg import RendererAgg
+from matplotlib.testing.decorators import image_comparison
+
+
+with warnings.catch_warnings():
+    warnings.simplefilter('ignore')
+    needs_usetex = pytest.mark.skipif(
+        not matplotlib.checkdep_usetex(True),
+        reason="This test needs a TeX installation")
 
 
 @image_comparison(baseline_images=['font_styles'])
@@ -31,7 +34,7 @@ def test_font_styles():
     from matplotlib.font_manager import FontProperties, findfont
     warnings.filterwarnings(
         'ignore',
-        ('findfont: Font family \[u?\'Foo\'\] not found. Falling back to .'),
+        r"findfont: Font family \[u?'Foo'\] not found. Falling back to .",
         UserWarning,
         module='matplotlib.font_manager')
 
@@ -136,7 +139,7 @@ def test_antialiasing():
     fig = plt.figure(figsize=(5.25, 0.75))
     fig.text(0.5, 0.75, "antialiased", horizontalalignment='center',
              verticalalignment='center')
-    fig.text(0.5, 0.25, "$\sqrt{x}$", horizontalalignment='center',
+    fig.text(0.5, 0.25, r"$\sqrt{x}$", horizontalalignment='center',
              verticalalignment='center')
     # NOTE: We don't need to restore the rcParams here, because the
     # test cleanup will do it for us.  In fact, if we do it here, it
@@ -175,7 +178,7 @@ def test_contains():
 
     # draw the text. This is important, as the contains method can only work
     # when a renderer exists.
-    plt.draw()
+    fig.canvas.draw()
 
     for x, y in zip(xs.flat, ys.flat):
         mevent.x, mevent.y = plt.gca().transAxes.transform_point([x, y])
@@ -233,19 +236,18 @@ def test_axes_titles():
     ax.set_title('right', loc='right', fontsize=12, fontweight=400)
 
 
-@cleanup
 def test_set_position():
     fig, ax = plt.subplots()
 
     # test set_position
     ann = ax.annotate(
         'test', (0, 0), xytext=(0, 0), textcoords='figure pixels')
-    plt.draw()
+    fig.canvas.draw()
 
     init_pos = ann.get_window_extent(fig.canvas.renderer)
     shift_val = 15
     ann.set_position((shift_val, shift_val))
-    plt.draw()
+    fig.canvas.draw()
     post_pos = ann.get_window_extent(fig.canvas.renderer)
 
     for a, b in zip(init_pos.min, post_pos.min):
@@ -254,12 +256,12 @@ def test_set_position():
     # test xyann
     ann = ax.annotate(
         'test', (0, 0), xytext=(0, 0), textcoords='figure pixels')
-    plt.draw()
+    fig.canvas.draw()
 
     init_pos = ann.get_window_extent(fig.canvas.renderer)
     shift_val = 15
     ann.xyann = (shift_val, shift_val)
-    plt.draw()
+    fig.canvas.draw()
     post_pos = ann.get_window_extent(fig.canvas.renderer)
 
     for a, b in zip(init_pos.min, post_pos.min):
@@ -287,11 +289,7 @@ def test_get_rotation_int():
 
 def test_get_rotation_raises():
     from matplotlib import text
-    import sys
-    if sys.version_info[:2] < (2, 7):
-        raise SkipTest("assert_raises as context manager "
-                       "not supported with Python < 2.7")
-    with assert_raises(ValueError):
+    with pytest.raises(ValueError):
         text.get_rotation('hozirontal')
 
 
@@ -371,7 +369,6 @@ def test_annotation_negative_fig_coords():
                 va='top')
 
 
-@cleanup
 def test_text_stale():
     fig, (ax1, ax2) = plt.subplots(1, 2)
     plt.draw_all()
@@ -393,3 +390,89 @@ def test_text_stale():
     assert not ax1.stale
     assert not ax2.stale
     assert not fig.stale
+
+
+@image_comparison(baseline_images=['agg_text_clip'],
+                  extensions=['png'])
+def test_agg_text_clip():
+    np.random.seed(1)
+    fig, (ax1, ax2) = plt.subplots(2)
+    for x, y in np.random.rand(10, 2):
+        ax1.text(x, y, "foo", clip_on=True)
+        ax2.text(x, y, "foo")
+
+
+def test_text_size_binding():
+    from matplotlib.font_manager import FontProperties
+
+    matplotlib.rcParams['font.size'] = 10
+    fp = FontProperties(size='large')
+    sz1 = fp.get_size_in_points()
+    matplotlib.rcParams['font.size'] = 100
+
+    assert sz1 == fp.get_size_in_points()
+
+
+@image_comparison(baseline_images=['font_scaling'],
+                  extensions=['pdf'])
+def test_font_scaling():
+    matplotlib.rcParams['pdf.fonttype'] = 42
+    fig, ax = plt.subplots(figsize=(6.4, 12.4))
+    ax.xaxis.set_major_locator(plt.NullLocator())
+    ax.yaxis.set_major_locator(plt.NullLocator())
+    ax.set_ylim(-10, 600)
+
+    for i, fs in enumerate(range(4, 43, 2)):
+        ax.text(0.1, i*30, "{fs} pt font size".format(fs=fs), fontsize=fs)
+
+
+@pytest.mark.parametrize('spacing1, spacing2', [(0.4, 2), (2, 0.4), (2, 2)])
+def test_two_2line_texts(spacing1, spacing2):
+    text_string = 'line1\nline2'
+    fig = plt.figure()
+    renderer = fig.canvas.get_renderer()
+
+    text1 = plt.text(0.25, 0.5, text_string, linespacing=spacing1)
+    text2 = plt.text(0.25, 0.5, text_string, linespacing=spacing2)
+    fig.canvas.draw()
+
+    box1 = text1.get_window_extent(renderer=renderer)
+    box2 = text2.get_window_extent(renderer=renderer)
+
+    # line spacing only affects height
+    assert box1.width == box2.width
+    if (spacing1 == spacing2):
+        assert box1.height == box2.height
+    else:
+        assert box1.height != box2.height
+
+
+def test_nonfinite_pos():
+    fig, ax = plt.subplots()
+    ax.text(0, np.nan, 'nan')
+    ax.text(np.inf, 0, 'inf')
+    fig.canvas.draw()
+
+
+def test_hinting_factor_backends():
+    plt.rcParams['text.hinting_factor'] = 1
+    fig = plt.figure()
+    t = fig.text(0.5, 0.5, 'some text')
+
+    fig.savefig(io.BytesIO(), format='svg')
+    expected = t.get_window_extent().intervalx
+
+    fig.savefig(io.BytesIO(), format='png')
+    # Backends should apply hinting_factor consistently (within 10%).
+    np.testing.assert_allclose(t.get_window_extent().intervalx, expected,
+                               rtol=0.1)
+
+
+@needs_usetex
+def test_single_artist_usetex():
+    # Check that a single artist marked with usetex does not get passed through
+    # the mathtext parser at all (for the Agg backend) (the mathtext parser
+    # currently fails to parse \frac12, requiring \frac{1}{2} instead).
+    fig, ax = plt.subplots()
+    ax.text(.5, .5, r"$\frac12$", usetex=True)
+    fig.canvas.draw()
