@@ -17,8 +17,8 @@ import six
 from six.moves import zip
 
 import numpy as np
+from matplotlib import rcParams
 
-from .mlab import dist
 from .patches import Circle, Rectangle, Ellipse
 from .lines import Line2D
 from .transforms import blended_transform_factory
@@ -80,7 +80,7 @@ class Widget(object):
         """
         return self._active
 
-    # set_active is overriden by SelectorWidgets.
+    # set_active is overridden by SelectorWidgets.
     active = property(get_active, lambda self, active: self.set_active(active),
                       doc="Is the widget active?")
 
@@ -140,22 +140,18 @@ class Button(AxesWidget):
     A GUI neutral button.
 
     For the button to remain responsive you must keep a reference to it.
+    Call :meth:`on_clicked` to connect to the button.
 
-    The following attributes are accessible
-
-      *ax*
+    Attributes
+    ----------
+    ax :
         The :class:`matplotlib.axes.Axes` the button renders into.
-
-      *label*
+    label :
         A :class:`matplotlib.text.Text` instance.
-
-      *color*
+    color :
         The color of the button when not hovering.
-
-      *hovercolor*
+    hovercolor :
         The color of the button when hovering.
-
-    Call :meth:`on_clicked` to connect to the button
     """
 
     def __init__(self, ax, label, image=None,
@@ -265,100 +261,88 @@ class Slider(AxesWidget):
     """
     A slider representing a floating point range.
 
-    For the slider to remain responsive you must maintain a
-    reference to it.
+    Create a slider from *valmin* to *valmax* in axes *ax*. For the slider to
+    remain responsive you must maintain a reference to it. Call
+    :meth:`on_changed` to connect to the slider event.
 
-    The following attributes are defined
-      *ax*        : the slider :class:`matplotlib.axes.Axes` instance
-
-      *val*       : the current slider value
-
-      *vline*     : a :class:`matplotlib.lines.Line2D` instance
-                     representing the initial value of the slider
-
-      *poly*      : A :class:`matplotlib.patches.Polygon` instance
-                     which is the slider knob
-
-      *valfmt*    : the format string for formatting the slider text
-
-      *label*     : a :class:`matplotlib.text.Text` instance
-                     for the slider label
-
-      *closedmin* : whether the slider is closed on the minimum
-
-      *closedmax* : whether the slider is closed on the maximum
-
-      *slidermin* : another slider - if not *None*, this slider must be
-                     greater than *slidermin*
-
-      *slidermax* : another slider - if not *None*, this slider must be
-                     less than *slidermax*
-
-      *dragging*  : allow for mouse dragging on slider
-
-    Call :meth:`on_changed` to connect to the slider event
+    Attributes
+    ----------
+    val : float
+        Slider value.
     """
     def __init__(self, ax, label, valmin, valmax, valinit=0.5, valfmt='%1.2f',
                  closedmin=True, closedmax=True, slidermin=None,
-                 slidermax=None, dragging=True, **kwargs):
+                 slidermax=None, dragging=True, valstep=None, **kwargs):
         """
-        Create a slider from *valmin* to *valmax* in axes *ax*.
-
-        Additional kwargs are passed on to ``self.poly`` which is the
-        :class:`matplotlib.patches.Rectangle` that draws the slider
-        knob.  See the :class:`matplotlib.patches.Rectangle` documentation for
-        valid property names (e.g., *facecolor*, *edgecolor*, *alpha*, ...).
-
         Parameters
         ----------
         ax : Axes
-            The Axes to put the slider in
+            The Axes to put the slider in.
 
         label : str
-            Slider label
+            Slider label.
 
         valmin : float
-            The minimum value of the slider
+            The minimum value of the slider.
 
         valmax : float
-            The maximum value of the slider
+            The maximum value of the slider.
 
-        valinit : float
-            The slider initial position
+        valinit : float, optional, default: 0.5
+            The slider initial position.
 
-        label : str
-            The slider label
+        valfmt : str, optional, default: "%1.2f"
+            Used to format the slider value, fprint format string.
 
-        valfmt : str
-            Used to format the slider value, fprint format string
+        closedmin : bool, optional, default: True
+            Indicate whether the slider interval is closed on the bottom.
 
-        closedmin : bool
-            Indicate whether the slider interval is closed on the bottom
+        closedmax : bool, optional, default: True
+            Indicate whether the slider interval is closed on the top.
 
-        closedmax : bool
-            Indicate whether the slider interval is closed on the top
-
-        slidermin : Slider or None
+        slidermin : Slider, optional, default: None
             Do not allow the current slider to have a value less than
-            `slidermin`
+            the value of the Slider `slidermin`.
 
-        slidermax : Slider or None
+        slidermax : Slider, optional, default: None
             Do not allow the current slider to have a value greater than
-            `slidermax`
+            the value of the Slider `slidermax`.
 
+        dragging : bool, optional, default: True
+            If True the slider can be dragged by the mouse.
 
-        dragging : bool
-            if the slider can be dragged by the mouse
+        valstep : float, optional, default: None
+            If given, the slider will snap to multiples of `valstep`.
 
+        Notes
+        -----
+        Additional kwargs are passed on to ``self.poly`` which is the
+        :class:`~matplotlib.patches.Rectangle` that draws the slider
+        knob.  See the :class:`~matplotlib.patches.Rectangle` documentation for
+        valid property names (e.g., `facecolor`, `edgecolor`, `alpha`).
         """
         AxesWidget.__init__(self, ax)
 
+        if slidermin is not None and not hasattr(slidermin, 'val'):
+            raise ValueError("Argument slidermin ({}) has no 'val'"
+                             .format(type(slidermin)))
+        if slidermax is not None and not hasattr(slidermax, 'val'):
+            raise ValueError("Argument slidermax ({}) has no 'val'"
+                             .format(type(slidermax)))
+        self.closedmin = closedmin
+        self.closedmax = closedmax
+        self.slidermin = slidermin
+        self.slidermax = slidermax
+        self.drag_active = False
         self.valmin = valmin
         self.valmax = valmax
+        self.valstep = valstep
+        valinit = self._value_in_bounds(valinit)
+        if valinit is None:
+            valinit = valmin
         self.val = valinit
         self.valinit = valinit
         self.poly = ax.axvspan(valmin, valinit, 0, 1, **kwargs)
-
         self.vline = ax.axvline(valinit, 0, 1, color='r', lw=1)
 
         self.valfmt = valfmt
@@ -383,11 +367,33 @@ class Slider(AxesWidget):
         self.cnt = 0
         self.observers = {}
 
-        self.closedmin = closedmin
-        self.closedmax = closedmax
-        self.slidermin = slidermin
-        self.slidermax = slidermax
-        self.drag_active = False
+        self.set_val(valinit)
+
+    def _value_in_bounds(self, val):
+        """ Makes sure self.val is with given bounds."""
+        if self.valstep:
+            val = np.round((val - self.valmin)/self.valstep)*self.valstep
+            val += self.valmin
+
+        if val <= self.valmin:
+            if not self.closedmin:
+                return
+            val = self.valmin
+        elif val >= self.valmax:
+            if not self.closedmax:
+                return
+            val = self.valmax
+
+        if self.slidermin is not None and val <= self.slidermin.val:
+            if not self.closedmin:
+                return
+            val = self.slidermin.val
+
+        if self.slidermax is not None and val >= self.slidermax.val:
+            if not self.closedmax:
+                return
+            val = self.slidermax.val
+        return val
 
     def _update(self, event):
         """update the slider position"""
@@ -410,30 +416,18 @@ class Slider(AxesWidget):
             self.drag_active = False
             event.canvas.release_mouse(self.ax)
             return
-
-        val = event.xdata
-        if val <= self.valmin:
-            if not self.closedmin:
-                return
-            val = self.valmin
-        elif val >= self.valmax:
-            if not self.closedmax:
-                return
-            val = self.valmax
-
-        if self.slidermin is not None and val <= self.slidermin.val:
-            if not self.closedmin:
-                return
-            val = self.slidermin.val
-
-        if self.slidermax is not None and val >= self.slidermax.val:
-            if not self.closedmax:
-                return
-            val = self.slidermax.val
-
-        self.set_val(val)
+        val = self._value_in_bounds(event.xdata)
+        if (val is not None) and (val != self.val):
+            self.set_val(val)
 
     def set_val(self, val):
+        """
+        Set slider value to *val*
+
+        Parameters
+        ----------
+        val : float
+        """
         xy = self.poly.xy
         xy[2] = val, 1
         xy[3] = val, 0
@@ -449,10 +443,19 @@ class Slider(AxesWidget):
 
     def on_changed(self, func):
         """
-        When the slider value is changed, call *func* with the new
-        slider position
+        When the slider value is changed call *func* with the new
+        slider value
 
-        A connection id is returned which can be used to disconnect
+        Parameters
+        ----------
+        func : callable
+            Function to call when slider is changed.
+            The function must accept a single float as its arguments.
+
+        Returns
+        -------
+        cid : int
+            Connection id (which can be used to disconnect *func*)
         """
         cid = self.cnt
         self.observers[cid] = func
@@ -460,21 +463,28 @@ class Slider(AxesWidget):
         return cid
 
     def disconnect(self, cid):
-        """remove the observer with connection id *cid*"""
+        """
+        Remove the observer with connection id *cid*
+
+        Parameters
+        ----------
+        cid : int
+            Connection id of the observer to be removed
+        """
         try:
             del self.observers[cid]
         except KeyError:
             pass
 
     def reset(self):
-        """reset the slider to the initial value if needed"""
+        """Reset the slider to the initial value"""
         if (self.val != self.valinit):
             self.set_val(self.valinit)
 
 
 class CheckButtons(AxesWidget):
     """
-    A GUI neutral radio button.
+    A GUI neutral set of check buttons.
 
     For the check buttons to remain responsive you must keep a
     reference to this object.
@@ -602,6 +612,12 @@ class CheckButtons(AxesWidget):
         for cid, func in six.iteritems(self.observers):
             func(self.labels[index].get_text())
 
+    def get_status(self):
+        """
+        returns a tuple of the status (True/False) of all of the check buttons
+        """
+        return [l1.get_visible() for (l1, l2) in self.lines]
+
     def on_clicked(self, func):
         """
         When the button is clicked, call *func* with button label
@@ -621,6 +637,311 @@ class CheckButtons(AxesWidget):
             pass
 
 
+class TextBox(AxesWidget):
+    """
+    A GUI neutral text input box.
+
+    For the text box to remain responsive you must keep a reference to it.
+
+    The following attributes are accessible:
+
+      *ax*
+        The :class:`matplotlib.axes.Axes` the button renders into.
+
+      *label*
+        A :class:`matplotlib.text.Text` instance.
+
+      *color*
+        The color of the text box when not hovering.
+
+      *hovercolor*
+        The color of the text box when hovering.
+
+    Call :meth:`on_text_change` to be updated whenever the text changes.
+
+    Call :meth:`on_submit` to be updated whenever the user hits enter or
+    leaves the text entry field.
+    """
+
+    def __init__(self, ax, label, initial='',
+                 color='.95', hovercolor='1', label_pad=.01):
+        """
+        Parameters
+        ----------
+        ax : matplotlib.axes.Axes
+            The :class:`matplotlib.axes.Axes` instance the button
+            will be placed into.
+
+        label : str
+            Label for this text box. Accepts string.
+
+        initial : str
+            Initial value in the text box
+
+        color : color
+            The color of the box
+
+        hovercolor : color
+            The color of the box when the mouse is over it
+
+        label_pad : float
+            the distance between the label and the right side of the textbox
+        """
+        AxesWidget.__init__(self, ax)
+
+        self.DIST_FROM_LEFT = .05
+
+        self.params_to_disable = [key for key in rcParams if u'keymap' in key]
+
+        self.text = initial
+        self.label = ax.text(-label_pad, 0.5, label,
+                             verticalalignment='center',
+                             horizontalalignment='right',
+                             transform=ax.transAxes)
+        self.text_disp = self._make_text_disp(self.text)
+
+        self.cnt = 0
+        self.change_observers = {}
+        self.submit_observers = {}
+
+        # If these lines are removed, the cursor won't appear the first
+        # time the box is clicked:
+        self.ax.set_xlim(0, 1)
+        self.ax.set_ylim(0, 1)
+
+        self.cursor_index = 0
+
+        # Because this is initialized, _render_cursor
+        # can assume that cursor exists.
+        self.cursor = self.ax.vlines(0, 0, 0)
+        self.cursor.set_visible(False)
+
+        self.connect_event('button_press_event', self._click)
+        self.connect_event('button_release_event', self._release)
+        self.connect_event('motion_notify_event', self._motion)
+        self.connect_event('key_press_event', self._keypress)
+        self.connect_event('resize_event', self._resize)
+        ax.set_navigate(False)
+        ax.set_facecolor(color)
+        ax.set_xticks([])
+        ax.set_yticks([])
+        self.color = color
+        self.hovercolor = hovercolor
+
+        self._lastcolor = color
+
+        self.capturekeystrokes = False
+
+    def _make_text_disp(self, string):
+        return self.ax.text(self.DIST_FROM_LEFT, 0.5, string,
+                            verticalalignment='center',
+                            horizontalalignment='left',
+                            transform=self.ax.transAxes)
+
+    def _rendercursor(self):
+        # this is a hack to figure out where the cursor should go.
+        # we draw the text up to where the cursor should go, measure
+        # and save its dimensions, draw the real text, then put the cursor
+        # at the saved dimensions
+
+        widthtext = self.text[:self.cursor_index]
+        no_text = False
+        if(widthtext == "" or widthtext == " " or widthtext == "  "):
+            no_text = widthtext == ""
+            widthtext = ","
+
+        wt_disp = self._make_text_disp(widthtext)
+
+        self.ax.figure.canvas.draw()
+        bb = wt_disp.get_window_extent()
+        inv = self.ax.transData.inverted()
+        bb = inv.transform(bb)
+        wt_disp.set_visible(False)
+        if no_text:
+            bb[1, 0] = bb[0, 0]
+        # hack done
+        self.cursor.set_visible(False)
+
+        self.cursor = self.ax.vlines(bb[1, 0], bb[0, 1], bb[1, 1])
+        self.ax.figure.canvas.draw()
+
+    def _notify_submit_observers(self):
+        for cid, func in six.iteritems(self.submit_observers):
+                func(self.text)
+
+    def _release(self, event):
+        if self.ignore(event):
+            return
+        if event.canvas.mouse_grabber != self.ax:
+            return
+        event.canvas.release_mouse(self.ax)
+
+    def _keypress(self, event):
+        if self.ignore(event):
+            return
+        if self.capturekeystrokes:
+            key = event.key
+
+            if(len(key) == 1):
+                self.text = (self.text[:self.cursor_index] + key +
+                             self.text[self.cursor_index:])
+                self.cursor_index += 1
+            elif key == "right":
+                if self.cursor_index != len(self.text):
+                    self.cursor_index += 1
+            elif key == "left":
+                if self.cursor_index != 0:
+                    self.cursor_index -= 1
+            elif key == "home":
+                self.cursor_index = 0
+            elif key == "end":
+                self.cursor_index = len(self.text)
+            elif(key == "backspace"):
+                if self.cursor_index != 0:
+                    self.text = (self.text[:self.cursor_index - 1] +
+                                 self.text[self.cursor_index:])
+                    self.cursor_index -= 1
+            elif(key == "delete"):
+                if self.cursor_index != len(self.text):
+                    self.text = (self.text[:self.cursor_index] +
+                                 self.text[self.cursor_index + 1:])
+
+            self.text_disp.remove()
+            self.text_disp = self._make_text_disp(self.text)
+            self._rendercursor()
+            self._notify_change_observers()
+            if key == "enter":
+                self._notify_submit_observers()
+
+    def set_val(self, val):
+        newval = str(val)
+        if self.text == newval:
+            return
+        self.text = newval
+        self.text_disp.remove()
+        self.text_disp = self._make_text_disp(self.text)
+        self._rendercursor()
+        self._notify_change_observers()
+        self._notify_submit_observers()
+
+    def _notify_change_observers(self):
+        for cid, func in six.iteritems(self.change_observers):
+            func(self.text)
+
+    def begin_typing(self, x):
+        self.capturekeystrokes = True
+        # disable command keys so that the user can type without
+        # command keys causing figure to be saved, etc
+        self.reset_params = {}
+        for key in self.params_to_disable:
+            self.reset_params[key] = rcParams[key]
+            rcParams[key] = []
+
+    def stop_typing(self):
+        notifysubmit = False
+        # because _notify_submit_users might throw an error in the
+        # user's code, we only want to call it once we've already done
+        # our cleanup.
+        if self.capturekeystrokes:
+            # since the user is no longer typing,
+            # reactivate the standard command keys
+            for key in self.params_to_disable:
+                rcParams[key] = self.reset_params[key]
+            notifysubmit = True
+        self.capturekeystrokes = False
+        self.cursor.set_visible(False)
+        self.ax.figure.canvas.draw()
+        if notifysubmit:
+            self._notify_submit_observers()
+
+    def position_cursor(self, x):
+        # now, we have to figure out where the cursor goes.
+        # approximate it based on assuming all characters the same length
+        if len(self.text) == 0:
+            self.cursor_index = 0
+        else:
+            bb = self.text_disp.get_window_extent()
+
+            trans = self.ax.transData
+            inv = self.ax.transData.inverted()
+            bb = trans.transform(inv.transform(bb))
+
+            text_start = bb[0, 0]
+            text_end = bb[1, 0]
+
+            ratio = (x - text_start) / (text_end - text_start)
+
+            if ratio < 0:
+                ratio = 0
+            if ratio > 1:
+                ratio = 1
+
+            self.cursor_index = int(len(self.text) * ratio)
+
+        self._rendercursor()
+
+    def _click(self, event):
+        if self.ignore(event):
+            return
+        if event.inaxes != self.ax:
+            self.stop_typing()
+            return
+        if not self.eventson:
+            return
+        if event.canvas.mouse_grabber != self.ax:
+            event.canvas.grab_mouse(self.ax)
+        if not self.capturekeystrokes:
+            self.begin_typing(event.x)
+        self.position_cursor(event.x)
+
+    def _resize(self, event):
+        self.stop_typing()
+
+    def _motion(self, event):
+        if self.ignore(event):
+            return
+        if event.inaxes == self.ax:
+            c = self.hovercolor
+        else:
+            c = self.color
+        if c != self._lastcolor:
+            self.ax.set_facecolor(c)
+            self._lastcolor = c
+            if self.drawon:
+                self.ax.figure.canvas.draw()
+
+    def on_text_change(self, func):
+        """
+        When the text changes, call this *func* with event.
+
+        A connection id is returned which can be used to disconnect.
+        """
+        cid = self.cnt
+        self.change_observers[cid] = func
+        self.cnt += 1
+        return cid
+
+    def on_submit(self, func):
+        """
+        When the user hits enter or leaves the submision box, call this
+        *func* with event.
+
+        A connection id is returned which can be used to disconnect.
+        """
+        cid = self.cnt
+        self.submit_observers[cid] = func
+        self.cnt += 1
+        return cid
+
+    def disconnect(self, cid):
+        """remove the observer with connection id *cid*"""
+        for reg in (self.change_observers, self.submit_observers):
+            try:
+                del reg[cid]
+            except KeyError:
+                pass
+
+
 class RadioButtons(AxesWidget):
     """
     A GUI neutral radio button.
@@ -710,7 +1031,8 @@ class RadioButtons(AxesWidget):
 
         def inside(p):
             pcirc = np.array([p.center[0], p.center[1]])
-            return dist(pclicked, pcirc) < p.radius
+            d = pclicked - pcirc
+            return np.sqrt(np.dot(d, d)) < p.radius
 
         for i, (p, t) in enumerate(zip(self.circles, self.labels)):
             if t.get_window_extent().contains(event.x, event.y) or inside(p):
@@ -938,13 +1260,10 @@ class Cursor(AxesWidget):
     def __init__(self, ax, horizOn=True, vertOn=True, useblit=False,
                  **lineprops):
         """
-        Add a cursor to *ax*.  If ``useblit=True``, use the backend-
-        dependent blitting features for faster updates (GTKAgg
-        only for now).  *lineprops* is a dictionary of line properties.
-
-        .. plot :: mpl_examples/widgets/cursor.py
+        Add a cursor to *ax*.  If ``useblit=True``, use the backend-dependent
+        blitting features for faster updates.  *lineprops* is a dictionary of
+        line properties.
         """
-        # TODO: Is the GTKAgg limitation still true?
         AxesWidget.__init__(self, ax)
 
         self.connect_event('motion_notify_event', self.onmove)
@@ -1366,8 +1685,7 @@ class SpanSelector(_SelectorWidget):
     Visually select a min/max range on a single axis and call a function with
     those values.
 
-    To guarantee that the selector remains responsive, keep a reference to
-    it.
+    To guarantee that the selector remains responsive, keep a reference to it.
 
     In order to turn off the SpanSelector, set `span_selector.active=False`. To
     turn it back on, set `span_selector.active=True`.
@@ -1386,7 +1704,7 @@ class SpanSelector(_SelectorWidget):
 
     useblit : bool, default is False
       If True, use the backend-dependent blitting features for faster
-      canvas updates. Only available for GTKAgg right now.
+      canvas updates.
 
     rectprops : dict, default is None
       Dictionary of :class:`matplotlib.patches.Patch` properties
@@ -1416,7 +1734,7 @@ class SpanSelector(_SelectorWidget):
                                      rectprops=rectprops)
     >>> fig.show()
 
-    See also: :ref:`widgets-span_selector`
+    See also: :doc:`/gallery/widgets/span_selector`
 
     """
 
@@ -1433,8 +1751,7 @@ class SpanSelector(_SelectorWidget):
         rectprops['animated'] = self.useblit
 
         if direction not in ['horizontal', 'vertical']:
-            msg = "direction must be in [ 'horizontal' | 'vertical' ]"
-            raise ValueError(msg)
+            raise ValueError("direction must be 'horizontal' or 'vertical'")
         self.direction = direction
 
         self.rect = None
@@ -1708,6 +2025,8 @@ class RectangleSelector(_SelectorWidget):
         a box or nothing between click and actual position by setting
 
         ``drawtype = 'line'``, ``drawtype='box'`` or ``drawtype = 'none'``.
+        Drawing a line would result in a line from vertex A to vertex C in
+        a rectangle ABCD.
 
         *spancoords* is one of 'data' or 'pixels'.  If 'data', *minspanx*
         and *minspanx* will be interpreted in the same coordinates as
@@ -1774,8 +2093,7 @@ class RectangleSelector(_SelectorWidget):
         self.minspany = minspany
 
         if spancoords not in ('data', 'pixels'):
-            msg = "'spancoords' must be one of [ 'data' | 'pixels' ]"
-            raise ValueError(msg)
+            raise ValueError("'spancoords' must be 'data' or 'pixels'")
 
         self.spancoords = spancoords
         self.drawtype = drawtype
@@ -2046,6 +2364,13 @@ class RectangleSelector(_SelectorWidget):
 
     @property
     def geometry(self):
+        """
+        Returns numpy.ndarray of shape (2,5) containing
+        x (``RectangleSelector.geometry[1,:]``) and
+        y (``RectangleSelector.geometry[0,:]``)
+        coordinates of the four corners of the rectangle starting
+        and ending in the top left corner.
+        """
         if hasattr(self.to_draw, 'get_verts'):
             xfm = self.ax.transData.inverted()
             y, x = xfm.transform(self.to_draw.get_verts()).T
@@ -2106,7 +2431,7 @@ class EllipseSelector(RectangleSelector):
             self.to_draw.width = 2 * a
             self.to_draw.height = 2 * b
         else:
-            rad = np.arange(31) * 12 * np.pi / 180
+            rad = np.deg2rad(np.arange(31) * 12)
             x = a * np.cos(rad) + center[0]
             y = b * np.sin(rad) + center[1]
             self.to_draw.set_data(x, y)
@@ -2126,26 +2451,17 @@ class EllipseSelector(RectangleSelector):
 
 
 class LassoSelector(_SelectorWidget):
-    """Selection curve of an arbitrary shape.
+    """
+    Selection curve of an arbitrary shape.
 
-    For the selector to remain responsive you must keep a reference to
-    it.
+    For the selector to remain responsive you must keep a reference to it.
 
-    The selected path can be used in conjunction with
-    :func:`~matplotlib.path.Path.contains_point` to select
-    data points from an image.
+    The selected path can be used in conjunction with `~.Path.contains_point`
+    to select data points from an image.
 
-    In contrast to :class:`Lasso`, `LassoSelector` is written with an interface
-    similar to :class:`RectangleSelector` and :class:`SpanSelector` and will
-    continue to interact with the axes until disconnected.
-
-    Parameters:
-
-    *ax* : :class:`~matplotlib.axes.Axes`
-        The parent axes for the widget.
-    *onselect* : function
-        Whenever the lasso is released, the `onselect` function is called and
-        passed the vertices of the selected path.
+    In contrast to `Lasso`, `LassoSelector` is written with an interface
+    similar to `RectangleSelector` and `SpanSelector`, and will continue to
+    interact with the axes until disconnected.
 
     Example usage::
 
@@ -2156,15 +2472,24 @@ class LassoSelector(_SelectorWidget):
             print(verts)
         lasso = LassoSelector(ax, onselect)
 
-     *button* is a list of integers indicating which mouse buttons should
-        be used for rectangle selection.  You can also specify a single
-        integer if only a single button is desired.  Default is *None*,
-        which does not limit which button can be used.
+    Parameters
+    ----------
+    ax : :class:`~matplotlib.axes.Axes`
+        The parent axes for the widget.
+    onselect : function
+        Whenever the lasso is released, the *onselect* function is called and
+        passed the vertices of the selected path.
+    button : List[Int], optional
+        A list of integers indicating which mouse buttons should be used for
+        rectangle selection. You can also specify a single integer if only a
+        single button is desired.  Default is ``None``, which does not limit
+        which button can be used.
 
         Note, typically:
-         1 = left mouse button
-         2 = center mouse button (scroll wheel)
-         3 = right mouse button
+
+        - 1 = left mouse button
+        - 2 = center mouse button (scroll wheel)
+        - 3 = right mouse button
 
     """
 
@@ -2212,6 +2537,219 @@ class LassoSelector(_SelectorWidget):
         self.update()
 
 
+class PolygonSelector(_SelectorWidget):
+    """Select a polygon region of an axes.
+
+    Place vertices with each mouse click, and make the selection by completing
+    the polygon (clicking on the first vertex). Hold the *ctrl* key and click
+    and drag a vertex to reposition it (the *ctrl* key is not necessary if the
+    polygon has already been completed). Hold the *shift* key and click and
+    drag anywhere in the axes to move all vertices. Press the *esc* key to
+    start a new polygon.
+
+    For the selector to remain responsive you must keep a reference to
+    it.
+
+    Parameters
+    ----------
+    ax : :class:`~matplotlib.axes.Axes`
+        The parent axes for the widget.
+    onselect : function
+        When a polygon is completed or modified after completion,
+        the `onselect` function is called and passed a list of the vertices as
+        ``(xdata, ydata)`` tuples.
+    useblit : bool, optional
+    lineprops : dict, optional
+        The line for the sides of the polygon is drawn with the properties
+        given by `lineprops`. The default is ``dict(color='k', linestyle='-',
+        linewidth=2, alpha=0.5)``.
+    markerprops : dict, optional
+        The markers for the vertices of the polygon are drawn with the
+        properties given by `markerprops`. The default is ``dict(marker='o',
+        markersize=7, mec='k', mfc='k', alpha=0.5)``.
+    vertex_select_radius : float, optional
+        A vertex is selected (to complete the polygon or to move a vertex)
+        if the mouse click is within `vertex_select_radius` pixels of the
+        vertex. The default radius is 15 pixels.
+
+    Examples
+    --------
+    :doc:`/gallery/widgets/polygon_selector_demo`
+    """
+
+    def __init__(self, ax, onselect, useblit=False,
+                 lineprops=None, markerprops=None, vertex_select_radius=15):
+        # The state modifiers 'move', 'square', and 'center' are expected by
+        # _SelectorWidget but are not supported by PolygonSelector
+        # Note: could not use the existing 'move' state modifier in-place of
+        # 'move_all' because _SelectorWidget automatically discards 'move'
+        # from the state on button release.
+        state_modifier_keys = dict(clear='escape', move_vertex='control',
+                                   move_all='shift', move='not-applicable',
+                                   square='not-applicable',
+                                   center='not-applicable')
+        _SelectorWidget.__init__(self, ax, onselect, useblit=useblit,
+                                 state_modifier_keys=state_modifier_keys)
+
+        self._xs, self._ys = [0], [0]
+        self._polygon_completed = False
+
+        if lineprops is None:
+            lineprops = dict(color='k', linestyle='-', linewidth=2, alpha=0.5)
+        lineprops['animated'] = self.useblit
+        self.line = Line2D(self._xs, self._ys, **lineprops)
+        self.ax.add_line(self.line)
+
+        if markerprops is None:
+            markerprops = dict(mec='k', mfc=lineprops.get('color', 'k'))
+        self._polygon_handles = ToolHandles(self.ax, self._xs, self._ys,
+                                            useblit=self.useblit,
+                                            marker_props=markerprops)
+
+        self._active_handle_idx = -1
+        self.vertex_select_radius = vertex_select_radius
+
+        self.artists = [self.line, self._polygon_handles.artist]
+        self.set_visible(True)
+
+    def _press(self, event):
+        """Button press event handler"""
+        # Check for selection of a tool handle.
+        if ((self._polygon_completed or 'move_vertex' in self.state)
+                and len(self._xs) > 0):
+            h_idx, h_dist = self._polygon_handles.closest(event.x, event.y)
+            if h_dist < self.vertex_select_radius:
+                self._active_handle_idx = h_idx
+        # Save the vertex positions at the time of the press event (needed to
+        # support the 'move_all' state modifier).
+        self._xs_at_press, self._ys_at_press = self._xs[:], self._ys[:]
+
+    def _release(self, event):
+        """Button release event handler"""
+        # Release active tool handle.
+        if self._active_handle_idx >= 0:
+            self._active_handle_idx = -1
+
+        # Complete the polygon.
+        elif (len(self._xs) > 3
+              and self._xs[-1] == self._xs[0]
+              and self._ys[-1] == self._ys[0]):
+            self._polygon_completed = True
+
+        # Place new vertex.
+        elif (not self._polygon_completed
+              and 'move_all' not in self.state
+              and 'move_vertex' not in self.state):
+            self._xs.insert(-1, event.xdata)
+            self._ys.insert(-1, event.ydata)
+
+        if self._polygon_completed:
+            self.onselect(self.verts)
+
+    def onmove(self, event):
+        """Cursor move event handler and validator"""
+        # Method overrides _SelectorWidget.onmove because the polygon selector
+        # needs to process the move callback even if there is no button press.
+        # _SelectorWidget.onmove include logic to ignore move event if
+        # eventpress is None.
+        if not self.ignore(event):
+            event = self._clean_event(event)
+            self._onmove(event)
+            return True
+        return False
+
+    def _onmove(self, event):
+        """Cursor move event handler"""
+        # Move the active vertex (ToolHandle).
+        if self._active_handle_idx >= 0:
+            idx = self._active_handle_idx
+            self._xs[idx], self._ys[idx] = event.xdata, event.ydata
+            # Also update the end of the polygon line if the first vertex is
+            # the active handle and the polygon is completed.
+            if idx == 0 and self._polygon_completed:
+                self._xs[-1], self._ys[-1] = event.xdata, event.ydata
+
+        # Move all vertices.
+        elif 'move_all' in self.state and self.eventpress:
+            dx = event.xdata - self.eventpress.xdata
+            dy = event.ydata - self.eventpress.ydata
+            for k in range(len(self._xs)):
+                self._xs[k] = self._xs_at_press[k] + dx
+                self._ys[k] = self._ys_at_press[k] + dy
+
+        # Do nothing if completed or waiting for a move.
+        elif (self._polygon_completed
+              or 'move_vertex' in self.state or 'move_all' in self.state):
+            return
+
+        # Position pending vertex.
+        else:
+            # Calculate distance to the start vertex.
+            x0, y0 = self.line.get_transform().transform((self._xs[0],
+                                                          self._ys[0]))
+            v0_dist = np.sqrt((x0 - event.x) ** 2 + (y0 - event.y) ** 2)
+            # Lock on to the start vertex if near it and ready to complete.
+            if len(self._xs) > 3 and v0_dist < self.vertex_select_radius:
+                self._xs[-1], self._ys[-1] = self._xs[0], self._ys[0]
+            else:
+                self._xs[-1], self._ys[-1] = event.xdata, event.ydata
+
+        self._draw_polygon()
+
+    def _on_key_press(self, event):
+        """Key press event handler"""
+        # Remove the pending vertex if entering the 'move_vertex' or
+        # 'move_all' mode
+        if (not self._polygon_completed
+                and ('move_vertex' in self.state or 'move_all' in self.state)):
+            self._xs, self._ys = self._xs[:-1], self._ys[:-1]
+            self._draw_polygon()
+
+    def _on_key_release(self, event):
+        """Key release event handler"""
+        # Add back the pending vertex if leaving the 'move_vertex' or
+        # 'move_all' mode (by checking the released key)
+        if (not self._polygon_completed
+                and
+                (event.key == self.state_modifier_keys.get('move_vertex')
+                 or event.key == self.state_modifier_keys.get('move_all'))):
+            self._xs.append(event.xdata)
+            self._ys.append(event.ydata)
+            self._draw_polygon()
+        # Reset the polygon if the released key is the 'clear' key.
+        elif event.key == self.state_modifier_keys.get('clear'):
+            event = self._clean_event(event)
+            self._xs, self._ys = [event.xdata], [event.ydata]
+            self._polygon_completed = False
+            self.set_visible(True)
+
+    def _draw_polygon(self):
+        """Redraw the polygon based on the new vertex positions."""
+        self.line.set_data(self._xs, self._ys)
+        # Only show one tool handle at the start and end vertex of the polygon
+        # if the polygon is completed or the user is locked on to the start
+        # vertex.
+        if (self._polygon_completed
+                or (len(self._xs) > 3
+                    and self._xs[-1] == self._xs[0]
+                    and self._ys[-1] == self._ys[0])):
+            self._polygon_handles.set_data(self._xs[:-1], self._ys[:-1])
+        else:
+            self._polygon_handles.set_data(self._xs, self._ys)
+        self.update()
+
+    @property
+    def verts(self):
+        """Get the polygon vertices.
+
+        Returns
+        -------
+        list
+            A list of the vertices of the polygon as ``(xdata, ydata)`` tuples.
+        """
+        return list(zip(self._xs[:-1], self._ys[:-1]))
+
+
 class Lasso(AxesWidget):
     """Selection curve of an arbitrary shape.
 
@@ -2222,16 +2760,15 @@ class Lasso(AxesWidget):
     Unlike :class:`LassoSelector`, this must be initialized with a starting
     point `xy`, and the `Lasso` events are destroyed upon release.
 
-    Parameters:
-
-    *ax* : :class:`~matplotlib.axes.Axes`
+    Parameters
+    ----------
+    ax : `~matplotlib.axes.Axes`
         The parent axes for the widget.
-    *xy* : array
+    xy : array
         Coordinates of the start of the lasso.
-    *callback* : function
+    callback : callable
         Whenever the lasso is released, the `callback` function is called and
         passed the vertices of the selected path.
-
     """
 
     def __init__(self, ax, xy, callback=None, useblit=True):
