@@ -1,4 +1,4 @@
-'''
+"""
 Colorbar toolkit with two classes and a function:
 
     :class:`ColorbarBase`
@@ -16,8 +16,8 @@ Colorbar toolkit with two classes and a function:
 The :meth:`~matplotlib.figure.Figure.colorbar` method uses :func:`make_axes`
 and :class:`Colorbar`; the :func:`~matplotlib.pyplot.colorbar` function
 is a thin wrapper over :meth:`~matplotlib.figure.Figure.colorbar`.
+"""
 
-'''
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
@@ -76,8 +76,8 @@ colormap_kw_doc = '''
                   used. An alternative
                   :class:`~matplotlib.ticker.Formatter` object may be
                   given instead.
-    *drawedges*   [ False | True ] If true, draw lines at color
-                  boundaries.
+    *drawedges*   bool
+                  Whether to draw lines at color boundaries.
     ===========   ====================================================
 
     The following will probably be useful only in the context of
@@ -132,9 +132,9 @@ keyword arguments:
 Additional keyword arguments are of two kinds:
 
   axes properties:
-%s
+    %s
   colorbar properties:
-%s
+    %s
 
 If *mappable* is a :class:`~matplotlib.contours.ContourSet`, its *extend*
 kwarg is included automatically.
@@ -175,7 +175,7 @@ the other hand, the shortest axis has a data limits of [1,2], whose
 unconventional value is to prevent underflow when log scale is used.
 ''' % (make_axes_kw_doc, colormap_kw_doc)
 
-docstring.interpd.update(colorbar_doc=colorbar_doc)
+#docstring.interpd.update(colorbar_doc=colorbar_doc)
 
 
 class CbarAxesLocator(object):
@@ -374,14 +374,13 @@ class ColorbarBase(cm.ScalarMappable):
         if format is None:
             if isinstance(self.norm, colors.LogNorm):
                 # change both axis for proper aspect
-                self.ax.xaxis.set_scale("log")
-                self.ax.yaxis.set_scale("log")
-                self.ax._update_transScale()
+                self.ax.set_xscale("log")
+                self.ax.set_yscale("log")
                 self.cbar_axis.set_minor_locator(ticker.NullLocator())
                 formatter = ticker.LogFormatter()
             else:
                 formatter = None
-        elif cbook.is_string_like(format):
+        elif isinstance(format, six.string_types):
             formatter = ticker.FormatStrFormatter(format)
         else:
             formatter = format  # Assume it is a Formatter
@@ -748,7 +747,6 @@ class Colorbar(ColorbarBase):
         # to make one object track another automatically.
         #tcolors = [col.get_colors()[0] for col in CS.collections]
         #tlinewidths = [col.get_linewidth()[0] for lw in CS.collections]
-        #print 'tlinewidths:', tlinewidths
         ColorbarBase.add_lines(self, CS.levels, tcolors, tlinewidths)
 
     def update_bruteforce(self, mappable):
@@ -766,7 +764,9 @@ class Colorbar(ColorbarBase):
 def make_axes(parent, **kw):
     '''
     Resize and reposition a parent axes, and return a child
-    axes suitable for a colorbar::
+    axes suitable for a colorbar
+
+    ::
 
         cax, kw = make_axes(parent, **kw)
 
@@ -808,20 +808,21 @@ def make_axes(parent, **kw):
     cax.set_aspect(aspect, anchor=anchor, adjustable='box')
     return cax, kw
 
-
+@docstring.Substitution(colorbar_doc)
 def colorbar(mappable, cax=None, ax=None, **kw):
     """
     Create a colorbar for a ScalarMappable instance.
 
     Documentation for the pylab thin wrapper:
-    %(colorbar_doc)s
+
+    %s
     """
     import matplotlib.pyplot as plt
     if ax is None:
         ax = plt.gca()
     if cax is None:
         cax, kw = make_axes(ax, **kw)
-    cax.hold(True)
+    cax._hold = True
     cb = Colorbar(cax, mappable, **kw)
 
     def on_changed(m):
