@@ -1,12 +1,19 @@
-#!/usr/local/bin/python2.7
+"""
+======
+Cursor
+======
 
+"""
 from matplotlib.widgets import Cursor
 import numpy as np
 import matplotlib.pyplot as plt
 
 
+# Fixing random state for reproducibility
+np.random.seed(19680801)
+
 fig = plt.figure(figsize=(8, 6))
-ax = fig.add_subplot(111, axisbg='#FFFFCC')
+ax = fig.add_subplot(111, facecolor='#FFFFCC')
 
 x, y = 4*(np.random.rand(2, 100) - .5)
 ax.plot(x, y, 'o')
